﻿using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BarRaider.SdTools;

namespace FocusVolumeControl
{
    internal class UIState
    {
        public static Dictionary<string, object> Build(ActiveAudioSessionWrapper session)
        {
            var volume = (int)(session.Volume.MasterVolume * 100);

            var valueThing = new Dictionary<string, string>()
            {
                { "value", $"{volume}%" },
                { "opacity", "0.5" }
            };
            var opacity = session.Volume.Mute ? 0.5f : 1;

            var payload = new Dictionary<string, object>()
                {
                    { "indicator", ValueWithOpacity(volume, opacity) },
                    { "value", ValueWithOpacity($"{volume}%", opacity ) },
                    { "title", session.DisplayName },
                };

            return payload;
        }

        public static Dictionary<string, object> ValueWithOpacity(object value, float opacity)
        {
            return new Dictionary<string, object>()
            {
                { "value", value },
                { "opacity", opacity }
            };
        }

        public static Dictionary<string, object> BuildWithImage(ActiveAudioSessionWrapper session)
        {
            var payload = Build(session);
            var opacity = session.Volume.Mute ? 0.5f : 1;

            var iconData = "";

            try
            {
                var icon = Icon.ExtractAssociatedIcon(session.ExecutablePath);
                iconData = Tools.ImageToBase64(icon.ToBitmap(), true);
            }
            catch 
            { 
                iconData = "Image/pluginIcon.png";
            }

            payload["icon"] = ValueWithOpacity(iconData, opacity);

            return payload;
        }

    }
}
